var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Directive, HostListener, Injector, Renderer } from '@angular/core';
import { FormControlName } from '@angular/forms';
import * as VMasker from 'vanilla-masker';
import { findInArray } from '../helpers';
import { DoubleAgentValidator } from '../validator.service';
/**
 * Applies a mask to user input following the configuration
 * passed-in the json-schema which originated the formControlName (if it is the case)
 *
 * @export
 * @class InputMaskDirective
 * @implements {OnInit}
 */
var InputMaskDirective = (function () {
    function InputMaskDirective(injector, renderer) {
        this.injector = injector;
        this.renderer = renderer;
        this.origiMaxLength = 0;
    }
    InputMaskDirective.prototype.ngAfterViewInit = function () {
        this.formControl = this.formControlName().control;
    };
    InputMaskDirective.prototype.formControlName = function () {
        return this.injector.get(FormControlName);
    };
    InputMaskDirective.prototype.doubleAgent = function () {
        return this.injector.get(DoubleAgentValidator);
    };
    InputMaskDirective.prototype.getUiMask = function (input) {
        var _this = this;
        var jsonProperty = this.formControl.jsonSchemaProperty;
        if (jsonProperty && jsonProperty['ui']) {
            if (Array.isArray(jsonProperty['ui']['mask'])) {
                var masksArray = jsonProperty['ui']['mask'];
                VMasker(input).unMask();
                if (jsonProperty.maxLength > 0 && this.origiMaxLength === 0) {
                    this.origiMaxLength = jsonProperty.maxLength;
                }
                if (jsonProperty.format) {
                    var formatRegex_1 = this.doubleAgent().getFormat(jsonProperty.format);
                    if (formatRegex_1) {
                        masksArray = masksArray.filter(function (mask) {
                            var hasFormat = mask.format && mask.format == jsonProperty.format;
                            if (hasFormat) {
                                mask.matcher = formatRegex_1;
                                _this.formControl.jsonSchemaProperty.maxLength = mask.maxLength;
                            }
                            return hasFormat;
                        });
                    }
                }
                var mask = findInArray(masksArray, function (item) {
                    if (!item.matcher && item.format) {
                        item.matcher = _this.doubleAgent().getFormat(item.format);
                    }
                    if (!item.matcher) {
                        throw new Error("[DoubleAgentValidator] A \"ui.matcher\" or \"ui.format\" is required to jsonSchema property \"" + _this.formControlName().name + "\"");
                    }
                    var found = new RegExp(item.matcher).test(input.value);
                    if (found && !jsonProperty.format &&
                        (_this.origiMaxLength > 0 && _this.origiMaxLength != _this.formControl.jsonSchemaProperty.maxLength)) {
                        _this.formControl.jsonSchemaProperty.maxLength = _this.origiMaxLength;
                    }
                    return found;
                });
                return mask ? mask['value'] : null;
            }
            else {
                return jsonProperty['ui'] ? jsonProperty['ui']['mask'] : null;
            }
        }
        return null;
    };
    InputMaskDirective.prototype.onInput = function (input) {
        // Write back to model
        if (input && input.value) {
            var mask = this.getUiMask(input);
            if (mask) {
                if (mask.match(/^currency/)) {
                    var precision = 2;
                    var matchCurrencyWithPrecision = mask.match(/currency\:(\d+)$/);
                    if (matchCurrencyWithPrecision) {
                        precision = Number.parseInt(matchCurrencyWithPrecision[1]);
                    }
                    VMasker(input).maskMoney({
                        precision: precision
                    });
                }
                else {
                    VMasker(input).maskPattern(mask);
                }
                if (this.formControl && this.formControl.jsonSchemaProperty.maxLength) {
                    this.renderer.setElementAttribute(input, 'maxlength', this.formControl.jsonSchemaProperty.maxLength.toString());
                }
                setTimeout(function () {
                    input.setSelectionRange(input.value.length, input.value.length);
                }, 0);
            }
            this.writeValue(input.value);
        }
    };
    InputMaskDirective.prototype.writeValue = function (value) {
        this.formControlName().control.setValue(value);
    };
    return InputMaskDirective;
}());
__decorate([
    HostListener('input', ['$event.target']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [HTMLInputElement]),
    __metadata("design:returntype", void 0)
], InputMaskDirective.prototype, "onInput", null);
InputMaskDirective = __decorate([
    Directive({
        selector: '[formControlName]'
    }),
    __metadata("design:paramtypes", [Injector,
        Renderer])
], InputMaskDirective);
export { InputMaskDirective };
//# sourceMappingURL=input-mask.directive.js.map