import { By } from '@angular/platform-browser';
import { InputMaskDirective } from './input-mask.directive';
export function getFormElement(fixture, controlName) {
    return fixture.debugElement.query(By.css("input[formcontrolname=" + controlName + "]"));
}
export function getMockForMask(fixture, options) {
    var element = options.element || getFormElement(fixture, options.fieldName);
    var maskDirective = element.injector.get(InputMaskDirective);
    if (maskDirective && options.maskFormat) {
        maskDirective.formControl.jsonSchemaProperty.format = options.maskFormat;
    }
    var mock = {
        maskDirective: maskDirective,
        element: element,
        input: element.nativeElement,
        triggerInput: function (inputValue, maskFormat) {
            if (maskFormat === void 0) { maskFormat = null; }
            if (maskFormat) {
                mock.maskDirective.formControl.jsonSchemaProperty.format = maskFormat;
            }
            else if (mock.maskDirective.formControl.jsonSchemaProperty.format) {
                delete mock.maskDirective.formControl.jsonSchemaProperty.format;
            }
            mock.input.value = inputValue;
            element.triggerEventHandler('input', { target: mock.input });
        }
    };
    return mock;
}
export function triggerInput(fixture, options) {
    fixture.detectChanges();
    var fieldWithMask = getMockForMask(fixture, options);
    fieldWithMask.triggerInput(options.value);
    fixture.detectChanges();
    return fieldWithMask;
}
//# sourceMappingURL=test-helpers.js.map