var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Injectable } from '@angular/core';
import { Validators } from '@angular/forms';
import { DoubleAgentValidator } from '../validator.service';
import * as _ from 'lodash';
var DoubleAgentFormControlValidatorBuilder = (function () {
    function DoubleAgentFormControlValidatorBuilder(doubleAgentValidator) {
        this.doubleAgentValidator = doubleAgentValidator;
    }
    /**
     * Returns the validator representing some attribute in the schema
     * which will be used to Angular 2 validates the user input
     *
     * @param {string} schemaName
     * @param {string} fieldName
     * @returns {ValidatorFn}
     *
     * @memberOf FormControlValidatorBuilder
     */
    DoubleAgentFormControlValidatorBuilder.prototype.build = function (schema, property) {
        var validators = [];
        if (schema.required && schema.required.indexOf(property) >= 0) {
            validators.push(Validators.required);
        }
        validators.push(this.buildAngularValidator(schema.id, property));
        return Validators.compose(validators);
    };
    /**
     * This creates a Angular validator corresponding a json schema and optionally
     * only for a given property
     *
     * @param {string} schema
     * @param {string} property
     * @returns {ValidatorFn}
     *
     * @memberOf ValidacaoService
     */
    DoubleAgentFormControlValidatorBuilder.prototype.buildAngularValidator = function (schemaName, propertyOrFormData) {
        var _this = this;
        return function (control) {
            var validationResult = {
                jsonSchema: null
            };
            // builds the data which will be validated
            var data = {};
            if (_.isString(propertyOrFormData)) {
                data[propertyOrFormData] = control.value;
                var jsonSchemaFormControl = control;
                if (jsonSchemaFormControl.jsonSchemaProperty &&
                    jsonSchemaFormControl.jsonSchemaProperty.ui &&
                    jsonSchemaFormControl.jsonSchemaProperty.ui.dependents && control.root && control.root['controls']) {
                    _.each(jsonSchemaFormControl.jsonSchemaProperty.ui.dependents, function (propertyName) {
                        data[propertyName] = control.root['controls'][propertyName].value;
                    });
                }
            }
            else {
                data = propertyOrFormData;
            }
            // runs the validation
            var result = _this.doubleAgentValidator.validate(schemaName, data);
            if (result.hasErrors) {
                // if a specific property was provided, then only returns error refering that property
                if (_.isString(propertyOrFormData)) {
                    var errorsOfProperty = _.filter(result.errors, function (error) {
                        return error.dataPath.match("." + propertyOrFormData);
                    });
                    if (errorsOfProperty.length > 0) {
                        validationResult.jsonSchema = {
                            errors: errorsOfProperty
                        };
                        return validationResult;
                    }
                }
                else {
                    // if no specific property was passed, so return all errors found
                    validationResult.jsonSchema = {
                        errors: result.errors
                    };
                    return validationResult;
                }
            }
            else {
                // retorna null no caso de não ter erros de validacao
                return null;
            }
        };
    };
    return DoubleAgentFormControlValidatorBuilder;
}());
DoubleAgentFormControlValidatorBuilder = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [DoubleAgentValidator])
], DoubleAgentFormControlValidatorBuilder);
export { DoubleAgentFormControlValidatorBuilder };
//# sourceMappingURL=form-control-validator-builder.service.js.map