import { FormBuilder, FormGroup, ValidatorFn } from '@angular/forms';
import { DoubleAgentValidator } from '../validator.service';
import { JsonSchema } from '../models/schema/json-schema';
import { DoubleAgentFormControlValidatorBuilder } from './form-control-validator-builder.service';
import { DoubleAgentFormGroup } from './form-group';
import { DoubleAgentFormArray, DoubleAgentControl } from './form-control';
export interface FormGroupStates {
    [key: string]: FormControlState;
}
export declare type FormControlState = string | {
    value: string;
    disabled: boolean;
};
/**
 * This class allows creates a formGroup which contains all the fields represented in an given schema
 * each one containing it's own angular validators
 * @export
 * @class DoubleAgentFormGroupBuilder
 */
export declare class DoubleAgentFormGroupBuilder {
    private doubleAgentValidator;
    private formControlValidatorBuilder;
    private formBuilder;
    /**
     * A json with values to "store" into FormGroup/FormArray's.
     *
     * These values are used to create initial FormGroups for
     * FormArrays collections
     */
    protected formValues: {
        [key: string]: any;
    };
    private config;
    /**
   * Creates an instance of FormGroupBuilder.
   *
   * @param {DoubleAgentValidator} doubleAgentValidator
   * @param {FormBuilder} formBuilder
   *
   * @memberOf FormGroupBuilder
   */
    constructor(doubleAgentValidator: DoubleAgentValidator, formControlValidatorBuilder: DoubleAgentFormControlValidatorBuilder, formBuilder: FormBuilder);
    setFormValues(data: {
        [key: string]: any;
    }): DoubleAgentFormGroupBuilder;
    /**
   * Builds a FormGroup containing all the attributes defined in the jsonSchema
   *
   * @param {string} schemaName
   * @param {object} config The object properties to override in `this.config` property
   * @returns {FormGroup}
   *
   * @memberOf FormGroupBuilder
   */
    build(schemaName: string, config?: {
        formGroupStates?: FormGroupStates;
        createValidators?: boolean;
        fillFormValues?: boolean;
        initializeEmptyCollections?: boolean;
    }): DoubleAgentFormGroup | FormGroup;
    /**
     * Add FormGroups for FormArrays, if `values`
     * param contains more data than 1
     *
     * @param formArray FormArray to add FormGroups
     * @param values Json data verify the amount, and create FormGroups and/or fill their values
     * @see DoubleAgentFormGroupBuilder.config.fillFormValues
     */
    addFormArrayValues(formArray: DoubleAgentFormArray, values: {
        [key: string]: any;
    }[]): DoubleAgentFormArray;
    private addKeywordsValidator(schema, formGroup);
    /**
     * Create controls and children FormGroup's from
     * a jsonSchema
     *
     * @param jsonSchema
     * @returns A object of controls/groups
     * @see DoubleAgentValidator.getSchema
     */
    createControls(jsonSchema: JsonSchema, values?: any): {
        [key: string]: DoubleAgentControl;
    };
    buildAngularFormGroupValidator(schemaName: string, keywords: string[], formGroup: FormGroup): ValidatorFn;
}
