var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Injectable } from '@angular/core';
import { FormBuilder, FormControl, FormGroup, FormArray } from '@angular/forms';
import { DoubleAgentValidator } from '../validator.service';
import * as _ from 'lodash';
import { DoubleAgentFormControlValidatorBuilder } from './form-control-validator-builder.service';
/**
 * This class allows creates a formGroup which contains all the fields represented in an given schema
 * each one containing it's own angular validators
 * @export
 * @class DoubleAgentFormGroupBuilder
 */
var DoubleAgentFormGroupBuilder = (function () {
    /**
   * Creates an instance of FormGroupBuilder.
   *
   * @param {DoubleAgentValidator} doubleAgentValidator
   * @param {FormBuilder} formBuilder
   *
   * @memberOf FormGroupBuilder
   */
    function DoubleAgentFormGroupBuilder(doubleAgentValidator, formControlValidatorBuilder, formBuilder) {
        this.doubleAgentValidator = doubleAgentValidator;
        this.formControlValidatorBuilder = formControlValidatorBuilder;
        this.formBuilder = formBuilder;
        this.config = {
            formGroupStates: {},
            createValidators: true,
            fillFormValues: false,
            initializeEmptyCollections: false
        };
    }
    DoubleAgentFormGroupBuilder.prototype.setFormValues = function (data) {
        this.formValues = data;
        return this;
    };
    /**
   * Builds a FormGroup containing all the attributes defined in the jsonSchema
   *
   * @param {string} schemaName
   * @param {object} config The object properties to override in `this.config` property
   * @returns {FormGroup}
   *
   * @memberOf FormGroupBuilder
   */
    DoubleAgentFormGroupBuilder.prototype.build = function (schemaName, config) {
        this.config.formGroupStates = config ? config.formGroupStates : {};
        this.config.createValidators = config ? config.createValidators : true;
        this.config.fillFormValues = config ? config.fillFormValues : true;
        this.config.initializeEmptyCollections = config ? config.initializeEmptyCollections : true;
        // TODO validar se o esquema existe e retornar erro apropriado
        var jsonSchema = this.doubleAgentValidator.getSchema(schemaName);
        // if the schema does not have the id, set with the schemaName argument
        if (!jsonSchema.id) {
            jsonSchema.id = schemaName;
        }
        if (!jsonSchema) {
            throw Error("Schema " + schemaName + " not found!");
        }
        // cria uma instância do FormGroup a partir da configuração construída
        var formGroupConfig = this.createControls(jsonSchema, this.formValues);
        var formGroup = this.formBuilder.group(formGroupConfig);
        // subscribe to valueChange of control if it has validateOnChange ui keyword
        _.each(formGroup.controls, function (formControl) {
            if (formControl.jsonSchemaProperty['ui'] && formControl.jsonSchemaProperty['ui']['validateOnChange']) {
                formControl.valueChanges.subscribe(function () {
                    _.each(formControl.jsonSchemaProperty['ui']['validateOnChange'], function (propertyToValidate) {
                        if (formGroup.controls[propertyToValidate]) {
                            formGroup.controls[propertyToValidate].updateValueAndValidity({ onlySelf: true });
                        }
                    });
                });
            }
        });
        if (this.config.createValidators) {
            // construir validador do FormGroup (keywords do objeto)
            this.addKeywordsValidator(jsonSchema, formGroup);
        }
        formGroup.jsonSchema = jsonSchema;
        // retorna o Form Group
        return formGroup;
    };
    /**
     * Add FormGroups for FormArrays, if `values`
     * param contains more data than 1
     *
     * @param formArray FormArray to add FormGroups
     * @param values Json data verify the amount, and create FormGroups and/or fill their values
     * @see DoubleAgentFormGroupBuilder.config.fillFormValues
     */
    DoubleAgentFormGroupBuilder.prototype.addFormArrayValues = function (formArray, values) {
        var _this = this;
        if (values && values.length > 0) {
            values.forEach(function (groupValues, index) {
                var group = formArray.at(index);
                if (!group) {
                    formArray.addGroup();
                }
                if (_this.config.fillFormValues) {
                    formArray.at(index).setValue(groupValues);
                }
            });
        }
        return formArray;
    };
    DoubleAgentFormGroupBuilder.prototype.addKeywordsValidator = function (schema, formGroup) {
        var keywords = this.doubleAgentValidator.getKeywords(schema);
        var validator = this.buildAngularFormGroupValidator(schema.id, keywords, formGroup);
        formGroup.setValidators([validator]);
    };
    /**
     * Create controls and children FormGroup's from
     * a jsonSchema
     *
     * @param jsonSchema
     * @returns A object of controls/groups
     * @see DoubleAgentValidator.getSchema
     */
    DoubleAgentFormGroupBuilder.prototype.createControls = function (jsonSchema, values) {
        var _this = this;
        var formGroupConfig = {};
        _.each(jsonSchema.properties, function (property, propertyName) {
            if (property.skip) {
                return;
            }
            if (property.items || property.group) {
                var collectionControls_1 = _this.createControls(property.items || property);
                var newGroup_1 = function (controls) {
                    if (controls === void 0) { controls = {}; }
                    controls = _.merge(_.cloneDeep(collectionControls_1), controls);
                    return new FormGroup(controls);
                };
                var customFormArray = (function (_super) {
                    __extends(class_1, _super);
                    function class_1(controls, validator, asyncValidator) {
                        return _super.call(this, controls, validator, asyncValidator) || this;
                    }
                    class_1.prototype.newGroup = function (controls) {
                        if (controls === void 0) { controls = {}; }
                        return newGroup_1(controls);
                    };
                    class_1.prototype.addGroup = function (controls) {
                        if (controls === void 0) { controls = {}; }
                        var group = this.newGroup(controls);
                        this.push(group);
                        return group;
                    };
                    return class_1;
                }(FormArray));
                if (property.type === 'array' && property.groupArray) {
                    var firstEmptyGroup = _this.config.initializeEmptyCollections ? [newGroup_1()] : [];
                    var formArray_1 = new customFormArray(firstEmptyGroup);
                    if (values) {
                        if (values instanceof Array) {
                            values.forEach(function (innerValues) {
                                if (innerValues[propertyName]) {
                                    _this.addFormArrayValues(formArray_1, innerValues[propertyName]);
                                }
                            });
                        }
                        else if (values[propertyName] instanceof Array) {
                            _this.addFormArrayValues(formArray_1, values[propertyName]);
                        }
                    }
                    formArray_1.jsonSchemaProperty = jsonSchema.properties[propertyName];
                    formArray_1.propertyName = propertyName;
                    formGroupConfig[propertyName] = formArray_1;
                }
                else {
                    var formGroup = newGroup_1();
                    formGroup.jsonSchemaProperty = jsonSchema.properties[propertyName];
                    formGroup.propertyName = propertyName;
                    formGroupConfig[propertyName] = formGroup;
                }
            }
            else {
                if (!_.has('$ref', propertyName) || !_.has('$ref', propertyName + ".properties") || property.type === 'object') {
                    // Igore the property if refer another object or is an object
                    var formState = '';
                    if (_this.config.formGroupStates && _this.config.formGroupStates[propertyName]) {
                        formState = _this.config.formGroupStates[propertyName];
                    }
                    var formControl = _this.config && _this.config.createValidators
                        ? new FormControl(formState, _this.formControlValidatorBuilder.build(jsonSchema, propertyName))
                        : new FormControl(formState);
                    formControl.jsonSchemaProperty = jsonSchema.properties[propertyName];
                    formControl.propertyName = propertyName;
                    formGroupConfig[propertyName] = formControl;
                }
            }
        });
        return formGroupConfig;
    };
    DoubleAgentFormGroupBuilder.prototype.buildAngularFormGroupValidator = function (schemaName, keywords, formGroup) {
        var _this = this;
        return function (control) {
            var validationResult = {
                jsonSchema: null,
            };
            var data = formGroup.value;
            // runs the validation
            var result = _this.doubleAgentValidator.validate(schemaName, data);
            if (result.hasErrors) {
                var errorsOfKeyword = _.filter(result.errors, function (error) {
                    return _.includes(keywords, error.keyword);
                });
                if (errorsOfKeyword.length > 0) {
                    validationResult.jsonSchema = {
                        errors: errorsOfKeyword,
                    };
                    return validationResult;
                }
            }
            // retorna null no caso de não ter erros de validacao
            return null;
        };
    };
    return DoubleAgentFormGroupBuilder;
}());
DoubleAgentFormGroupBuilder = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [DoubleAgentValidator,
        DoubleAgentFormControlValidatorBuilder,
        FormBuilder])
], DoubleAgentFormGroupBuilder);
export { DoubleAgentFormGroupBuilder };
//# sourceMappingURL=form-group-builder.service.js.map