var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { DoubleAgentValidator } from './validator.service';
import { ValidatorDefinitionsLoader } from './definitions-loader.service';
import { Injectable } from '@angular/core';
import { RemoteLoader } from './models/remote-loader';
/**
 *
 * This classs provide a facility to load json schema definitions into a DoubleAgentValidator instance and provide it
 * to Angular Dependency Injection
 * @export
 * @class DoubleAgentValidatorNg2Factory
 */
var DoubleAgentValidatorNg2Factory = (function () {
    /**
     * Creates an instance of DoubleAgentValidatorNg2Factory.
     *
     * @param {DoubleAgentValidator} doubleAgentValidator
     * @param {remoteLoader} the remoteLoader which will be used to load the validation scripts
     *
     * @memberOf DoubleAgentValidatorNg2Factory
     */
    function DoubleAgentValidatorNg2Factory(doubleAgentValidator, remoteLoader) {
        this.doubleAgentValidator = doubleAgentValidator;
        this.remoteLoader = remoteLoader;
    }
    /**
     * This static function is utilized as a provider Factory to builds the DoubleAgentValidator
     * instance filled with json schemas from an given url into the Angular2  dependency injection
     *
     * @static
     * @param {Injector} injector
     * @param {DoubleAgentValidatorNg2Factory} factory
     * @returns {Promise<void>}
     *
     * @memberOf DoubleAgentValidatorNg2Factory
     */
    DoubleAgentValidatorNg2Factory.factoryFn = function (injector, factory, urlSchema) {
        return new Promise(function (resolve, reject) {
            var errors = null;
            if (urlSchema == null) {
                errors = 'DoubleAgentValidator Module needs an url provided through the DOUBLE_AGENT_VALIDATOR_SCHEMA_URL token';
            }
            if (errors) {
                reject(errors);
            }
            factory.load(urlSchema).then(function () { return resolve(); }).catch(function () { return reject(); });
        });
        // }
    };
    /**
     * Loads a script from a url, parses it and load into the ajv object.
     * At this moment is using a iframe to isolate the parse/evaluate of the code.
     * Maybe it would useful have a strategy loading using web worker
     * @param {string} url
     * @returns {Promise<void>}
     *
     * @memberOf DoubleAgentValidatorNg2Factory
     */
    DoubleAgentValidatorNg2Factory.prototype.load = function (url) {
        var _this = this;
        var validationsLoader = new ValidatorDefinitionsLoader(this.remoteLoader);
        var iframe = document.createElement('iframe');
        iframe.id = 'DoubleAgentValidator';
        iframe.src = 'about:blank';
        iframe.style.border = '0';
        iframe.style.background = 'transparent';
        iframe.style.display = 'none';
        iframe.style.width = '1px';
        iframe.style.height = '1px';
        iframe.sandbox = 'allow-scripts allow-same-origin allow-modals';
        document.body.appendChild(iframe);
        var window = iframe.contentWindow;
        return new Promise(function (resolve, reject) {
            validationsLoader.load(window, url).then(function () {
                _this.doubleAgentValidator['scriptContext'] = window;
                window['DoubleAgentValidator'] = _this.doubleAgentValidator;
                // this.doubleAgentValidator['_notifyReady']();
                resolve(null);
            }, function (e) {
                reject('Could not create the DoubleAgentValidator instance: ' + e);
            });
        });
    };
    return DoubleAgentValidatorNg2Factory;
}());
DoubleAgentValidatorNg2Factory = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [DoubleAgentValidator,
        RemoteLoader])
], DoubleAgentValidatorNg2Factory);
export { DoubleAgentValidatorNg2Factory };
//# sourceMappingURL=ng2-factory.service.js.map